;===============================;
;Initialization Code		;
;===============================;
;Assembled with sekas v1.00	;
;===============================;
;This code initializes the hard	;
;ware and loads the z80 with a	;
;program that (presumably) does	;
;nothing. This code was lifted	;
;from a post on spirtesmind.	;
;===============================;

incsrc header.asm			;ROM header and interrupt table

;print "EntryPoint @ $",pc

EntryPoint:
	tst.l	($a10008)
	bne	SkipJoyDetect
	
	tst.w	($a1000c)

SkipJoyDetect:
	bne	SkipSetup
	
	lea	Table(pc),a5
	movem.w	(a5)+,d5-d7
	movem.l	(a5)+,a0-a4
	
CheckVersion:
	move.b	($a10001),d0
	andi.b	#$0f,d0
	beq	WrongVersion

!SEGA	=	#$53454741		;"SEGA" in ASCII
	
	move.l	!SEGA,$2f00(a1)		;Sega Security Code

WrongVersion:

	move.w	(a4),d0			;Read VDP Status
	moveq.l	#0,d0
	movea.l	d0,a6

	move.l	a6,usp			;Clear User Stack Pointer
	
	moveq.l	#$17,d1			;Initialize all 23 VDP registers
	
InitVDPRegister:
	move.b	(a5)+,d5		;d5 contains a mask since bit 17 is always set
	move.w	d5,(a4)			;for register setting
	add.w	d7,d5			;d7 contains a constant $0100 to increment Register No. value
	dbfa	d1,InitVDPRegister
	
	move.l	(a5)+,(a4)		;DMA fill...
	move.w	d0,(a3)			;...with value 0
	
	move.w	d7,(a1)			;move.w	#$0100,($a11100)	Z80 BUSREQ
	move.w	d7,(a2)			;move.w	#$0100,($a11200)	Z80 RESET
	
waitBUSREQ:
	btst.b	d0,(a1)			;while (BUSREQ & 0X0 != 0)
	bne	waitBUSREQ

ResetYM2612:
	move.w	#$4000,d1
	moveq	#26,d2
	
ResetYM2612Loop1:
	move.b	(a5)+,d1		;FM reg
	move.b	(a5)+,$0(a0,d1.w)	;FM data

	nop
	nop

	dbfa	d2,ResetYM2612Loop1

	moveq	#$30,d1
	moveq	#$5f,d2
	
ResetYM2612Loop2:
	move.b	d1,$4000(a0)		;FM reg
	nop
	nop

	move.b	#$ff,$4001(a0)		;FM data
	nop
	nop
	
	move.b	d1,$4002(a0)		;FM reg
	nop
	nop
	
	move.b	#$ff,$4003(a0)		;FM data
	nop
	nop
	
	addq.b	#1,d1
	dbfa	d2,ResetYM2612Loop2
	
ResetPSG:
	move.b	(a5)+,$11(a3)
	move.b	(a5)+,$11(a3)
	move.b	(a5)+,$11(a3)
	move.b	(a5)+,$11(a3)
	
;	moveq	#$25,d2			;Length of Z80 init program

;LoadZ80:
;	move.b	(a5)+,(a0)+
;	dbfa	d2,LoadZ80
	
	move.w	d0,(a2)			;move.w	#0,($a11200)	Z80 RESET
	move.w	d0,(a1)			;move.w	#0,($a11100)	Z80 BUSREQ
	move.w	d7,(a2)			;move.w	#$0100,($a11200)	Z80 RESET
	
ClearRAM:
	move.l	d0,-(a6)
	dbfa	d6,ClearRAM
	
	move.l	(a5)+,(a4)		;Turn DMA OFF, Auto-Increment 1
	move.l	(a5)+,(a4)
	
	moveq.l	#$1f,d3

ClearCRAM:
	move.l	d0,(a3)
	dbra	d3,ClearCRAM
	
	move.l	(a5)+,(a4)
	moveq.l	#$13,d4

ClearVSRAM:
	move.l	d0,(a3)
	dbfa	d4,ClearVSRAM
	
	move.w	d0,(a2)			;Turn on Z80
	movem.l	(a6),d0-d7/a0-a6	;Clear all registers
	move.w	#$2700,sr		;Enable all interrupts
	
SkipSetup:
	bra	Continue

;print "Table @ $",pc
	
Table:
	dw	$8000			;High bit always set for VDP register set
	dw	$3fff			;(RAM_MAX / 4) - 1
	dw	$0100			;Constant to increment VDP register number by

	dd	$a00000			;Sound RAM
	dd	$a11100			;Z80 BusReq
	dd	$a11200			;Z80 Reset

	dd	$c00000			;VRAM DATA PORT
	dd	$c00004			;VRAM CTRL PORT

;print "InitVDPRegVals @ $",pc
	
InitVDPRegVals:
	db	$04			;$8004	Palette Select
	db	$14			;$8114	DMA, Genesis
	db	$30			;$8230	A Plane @ $C000
	db	$2c			;$832c	Window @ $B000
	db	$07			;$8407	B Plane @ $E000
	db	$54			;$8554	Sprite ATB @ $A800
	db	$00			;$8600	Unused
	db	$00			;$8700	B/G Color = 0
	db	$00			;$8800	Unused
	db	$00			;$8900	Unused
	db	$00			;$8a00	H int = 0
	db	$00			;$8b00	Full V, Full H scroll
	db	$81			;$8c81	40 cells width
	db	$2b			;$8d2b	H Scroll @ $AC00
	db	$00			;$8e00	Unused
	db	$01			;$8f01	Auto-incrment 1
	db	$01			;$9001	64 x 32
	db	$00			;$9100	No window
	db	$00			;$9200	No window
	db	$ff			;$93ff	DMA length --FF
	db	$ff			;$94ff	DMA lenght FF--
	db	$00			;$9500	DMA src ----00
	db	$00			;$9600	DMA src --00--
	db	$80			;$9780	DMA sr 80-----, DMA Fill
	
	dd	$40000080		;VRAM Fill @ $0000

;print "FMReset @ $",pc

FMReset:
	;Disable LFO
	db	0, $22
	db	1, $00
	
	;Disable timer & set channel 6 to normal mode
	db	0, $27
	db	1, $00
	
	;All KEY_OFF
	db	0, $28
	db	1, $00
	db	1, $04
	db	1, $01
	db	1, $05
	db	1, $02
	db	1, $06
	
	;Disable DAC
	db	0, $2a
	db	1, $80
	db	0, $2b
	db	1, $00
	
	;Turn Off channels
	db	0, $b4
	db	1, $00
	db	0, $b5
	db	1, $00
	db	0, $b6
	db	1, $00
	db	2, $b4
	db	3, $00
	db	2, $b5
	db	3, $00
	db	2, $b6
	db	3, $00

PSGReset:			;PSG Register initialization values
	db	$9f		;Set ch0 attenuation to max
	db	$bf		;Set ch1 attenuation to max
	db	$df		;Set ch2 attenuation to max
	db	$ff		;Set ch3 attenuation to max
	
	dd	$81048f01		;Turn DMA OFF, Auto-inc 1
	dd	$c0000000		;CRAM Write
	dd	$40000010		;VSRAM Write

Continue:
	tst.w	($c00004)
	movea.l	($00000000),a7		;set stack pointer